Documentation - Workers
====================

After [queues are configured](2.Configuration.md), [jobs classes are written](3.Jobs.md) and pushed into a queue, they
must be popped out by a worker to be executed. This is done, as explained in the [introduction](1.Introduction.md), by
workers. A worker for SlmQueue is a class implementing the `SlmQueue\Worker\WorkerInterface`.

A worker is started by by calling its `processQueue($queue, array $options = array())` method. The `$queue` parameter is
a string and should be the service name of the queue as registered in the queue plugin manager. The `$options` parameter
is an optional array which is directly passed on to the `pop()` method of the queue. This can contain flags specifically
for the different queue implementations.

Worker stop conditions
----------------------

The worker will be a long running call, due to the `while(true){ /*...*/ }` loop it contains inside the `processQueue()`
method. However, there are reasons to cancel the loop and stop the worker to continue. PHP is not the best language for
creating infinite running scripts.

It is wise to abort the script frequently, for example after x number of cycles in the `while` loop. A worker is
initiated with a `WorkerOptions` class, where it is possible to set a maximum number of runs. In this same options class,
a maximum memory level can be set. If you have accidentally a memory leak, this option can make sure the script aborts
eventually after reaching this level.

Both options can be set at the `worker` key in the SlmQueue configuration:

```php
'slm_queue' => array(
    'worker' => array(
        'max_runs'   => 100000,           // 10,000 runs
        'max_memory' => 20 * 1024 * 1024, // 20 MB
    )
),
```

Secondly, it is possible to catch a stop condition under Linux-like systems (as well as OS X). If a worker is started
from the command line interface (CLI), it is possible to send a SIGTERM or SiGINT call to the worker. SlmQueue is smart 
enough not to quit the script diretly, but let the job finish its work first and then break out of the loop. For Windows
users this option is not available.

Command line utility
--------------------

SlmQueue provides integration with the Zend Framework 2 console routes and ease the use of workers. You can start a
worker for e.g. the beanstalkd queue system by running the following command*:

```php
php public/index.php queue beanstalkd <queue-name>
```

*) assumed here is your working directory is the application root and your application is structured like the Zend
Skeleton Application with public directory and an `index.php` file.

This CLI command dispatches a SlmQueue controller. A worker is injected into the controller and it simply calls the
worker's `processQueue()` method. Every queue implementation has it's own CLI command so read the documentation for
these implementations for the exact command.

Navigation
----------

Previous page: [Queue Aware](4.QueueAware.md)
Next page: [Events](6.Events.md)

1. [Introduction](1.Introduction.md)
2. [Configuration](2.Configuration.md)
3. [Jobs](3.Jobs.md)
4. [QueueAware](4.QueueAware.md)
5. [Workers](5.Workers.md)
6. [Events](6.Events.md)
7. [Worker management](7.WorkerManagement.md)
